@@ -20,11 +20,18 @@ struct DialOutContext {
 
 MainWindow::MainWindow (QWidget *parent)
 : QDeclarativeView (parent)
+, fLogfile (this)
 , icoGoogle (":/Google.png")
 , pSystray (NULL)
 , pContactsView (NULL)
 , pInboxView (NULL)
 , pWebWidget (new WebWidget (this, Qt::Window))
+#ifdef Q_WS_MAEMO_5
+, infoBox (this)
+#endif
+, menuFile ("&File", this)
+, actLogin ("Login...", this)
+, actExit ("Exit", this)
 , bLoggedIn (false)
 , modelRegNumber (this)
 , indRegPhone (0)
@@ -37,6 +44,16 @@ MainWindow::MainWindow (QWidget *parent)
                      this                   , SLOT  (orientationChanged()));
 #endif
 
+    // Initialize logging
+    QString strLogfile = QDir::homePath ();
+    if (!strLogfile.endsWith (QDir::separator ()))
+    {
+        strLogfile += QDir::separator ();
+    }
+    strLogfile += "qgvdial.log";
+    fLogfile.setFileName (strLogfile);
+    fLogfile.open (QIODevice::WriteOnly | QIODevice::Append);
+
     // This must be done at least once so that the initial qml is loaded.
     // Even if it is desktop, this must be done: The function takes care of
     // making it portrait for non-maemo.
@@ -70,8 +87,6 @@ MainWindow::MainWindow (QWidget *parent)
 
 MainWindow::~MainWindow ()
 {
-    //@@UV: Fix this
-//    delete ui;
 }//MainWindow::~MainWindow
 
 void
@@ -92,10 +107,10 @@ MainWindow::log (const QString &strText, int level /*= 10*/)
     cout << strLog.toStdString () << endl;
 
     // Send to log file
-//    if (fLogfile.isOpen ()) {
-//        QTextStream streamLog(&fLogfile);
-//        streamLog << strLog << endl;
-//    }
+    if (fLogfile.isOpen ()) {
+        QTextStream streamLog(&fLogfile);
+        streamLog << strLog << endl;
+    }
 }//MainWindow::log
 
 void
@@ -103,6 +118,26 @@ MainWindow::setStatus(const QString &strText, int timeout /* = 0*/)
 {
     //@@UV: Fix
     qDebug () << strText;
+
+#ifdef Q_WS_MAEMO_5
+    infoBox.hide ();
+    QLabel *theLabel = (QLabel *) infoBox.widget ();
+    if (NULL == theLabel) {
+        theLabel = new QLabel (strText, &infoBox);
+        theLabel->setAlignment (Qt::AlignHCenter);
+        infoBox.setWidget (theLabel);
+    } else {
+        theLabel->setText (strText);
+    }
+    infoBox.setTimeout (0 == timeout?3000:timeout);
+    infoBox.show ();
+#else
+    if (NULL != pSystray) {
+        pSystray->showMessage ("Status", strText,
+                               QSystemTrayIcon::Information,
+                               timeout);
+    }
+#endif
 }//MainWindow::setStatus
 
 void
@@ -166,6 +201,16 @@ MainWindow::init ()
     // Additional UI initializations:
     //@@UV: Need this for later
 //    ui->edNumber->setValidator (new PhoneNumberValidator (ui->edNumber));
+    actLogin.setShortcut (QKeySequence(Qt::CTRL + Qt::Key_L));
+    actExit.setShortcut (QKeySequence(Qt::CTRL + Qt::Key_Q));
+    menuFile.addAction (&actLogin);
+    menuFile.addAction (&actExit);
+    this->addAction (&actLogin);
+    this->addAction (&actExit);
+    QObject::connect (&actLogin, SIGNAL (triggered()),
+                       this    , SLOT   (on_action_Login_triggered()));
+    QObject::connect (&actExit, SIGNAL (triggered()),
+                       this   , SLOT   (on_actionE_xit_triggered()));
 
     // If the cache has the username and password, begin login
     if (dbMain.getUserPass (strUser, strPass))
@@ -206,6 +251,9 @@ MainWindow::doLogin ()
             break;
         }
 
+        OsDependent &osd = Singletons::getRef().getOSD ();
+        osd.setLongWork (this, true);
+
         bOk = true;
     } while (0); // End cleanup block (not a loop)
 
@@ -266,6 +314,9 @@ MainWindow::loginCompleted (bool bOk, const QVariantList &varList)
 
         QVariantList l;
         logoutCompleted (true, l);
+
+        OsDependent &osd = Singletons::getRef().getOSD ();
+        osd.setLongWork (this, false);
     }
     else
     {
@@ -280,8 +331,7 @@ MainWindow::loginCompleted (bool bOk, const QVariantList &varList)
         initInboxWidget ();
 
         // Allow access to buttons and widgets
-        //@@UV: Fix this later
-//        ui->action_Login->setText ("Logout");
+        actLogin.setText ("Logout");
         bLoggedIn = true;
 
         // Save the user name and password that was used to login
@@ -346,6 +396,9 @@ MainWindow::doLogout ()
     QVariantList l;
     webPage.enqueueWork (GVAW_logout, l, this,
                          SLOT (logoutCompleted (bool, const QVariantList &)));
+
+    OsDependent &osd = Singletons::getRef().getOSD ();
+    osd.setLongWork (this, true);
 }//MainWindow::doLogout
 
 void
@@ -357,12 +410,13 @@ MainWindow::logoutCompleted (bool, const QVariantList &)
 
     arrNumbers.clear ();
 
-    //@UV: Fix this later
-//    ui->action_Login->setText ("Login...");
+    actLogin.setText ("Login...");
 
     bLoggedIn = false;
 
     setStatus ("Logout complete");
+    OsDependent &osd = Singletons::getRef().getOSD ();
+    osd.setLongWork (this, false);
 }//MainWindow::logoutCompleted
 
 void
@@ -973,17 +1027,6 @@ MainWindow::on_btnHistory_clicked ()
     }
 }//MainWindow::on_btnHistory_clicked
 
-void
-MainWindow::keyPressEvent (QKeyEvent *event)
-{
-    if ((Qt::Key_Q == event->key ()) &&
-        (Qt::ControlModifier == event->modifiers ())) {
-        on_actionE_xit_triggered ();
-    } else {
-        QDeclarativeView::keyPressEvent (event);
-    }
-}//MainWindow::keyPressEvent
-
 void
 MainWindow::closeEvent (QCloseEvent *event)
 {
@@ -1005,9 +1048,6 @@ MainWindow::refreshRegisteredNumbers ()
             break;
         }
 
-        //@@UV: Fix this
-//        ui->cbDialMethod->clear ();
-//        ui->cbDialMethod->setEnabled (false);
         arrNumbers.clear ();
 
         QVariantList l;
@@ -1245,4 +1285,7 @@ MainWindow::onRegPhoneSelectionChange (int index)
         data.strName = "<Unknown>";
     }
     ctx->setContextProperty ("currentPhoneName", data.strName);
+
+    OsDependent &osd = Singletons::getRef().getOSD ();
+    osd.setLongWork (this, false);
 }//MainWindow::onRegPhoneSelectionChange
